/*
 * Copyright (c) 2017 yvolk (Yuri Volkov), http://yurivolkov.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.andstatus.app.net.social;

import android.support.test.InstrumentationRegistry;

import org.andstatus.app.context.MyContextHolder;
import org.andstatus.app.context.TestSuite;
import org.andstatus.app.data.MyContentType;
import org.andstatus.app.util.RawResourceUtils;
import org.andstatus.app.util.TriState;
import org.andstatus.app.util.UriUtils;
import org.junit.Before;
import org.junit.Test;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import static org.andstatus.app.context.DemoData.demoData;
import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;

public class ConnectionMastodonTest {
    private ConnectionMastodonMock connection;
    private String accountUserOid;

    @Before
    public void setUp() throws Exception {
        TestSuite.initializeWithData(this);
        accountUserOid = demoData.MASTODON_TEST_ACCOUNT_USER_OID;
        connection = new ConnectionMastodonMock();
    }

    @Test
    public void testGetHomeTimeline() throws IOException {
        String jso = RawResourceUtils.getString(InstrumentationRegistry.getInstrumentation().getContext(),
                org.andstatus.app.tests.R.raw.mastodon_home_timeline);
        connection.getHttpMock().setResponse(jso);

        List<MbActivity> timeline = connection.getTimeline(Connection.ApiRoutineEnum.HOME_TIMELINE,
                new TimelinePosition("2656388"), TimelinePosition.EMPTY, 20, accountUserOid);
        assertNotNull("timeline returned", timeline);
        int size = 1;
        assertEquals("Number of items in the Timeline", size, timeline.size());

        int ind = 0;
        MbActivity activity = timeline.get(ind);
        assertEquals("Timeline position", "22", activity.getTimelinePosition().getPosition());
        assertEquals("Message Oid", "22", activity.getMessage().oid);
        assertEquals("Account unknown " + activity, true, MyContextHolder.get().persistentAccounts().fromUser(activity.accountUser).isValid());
        MbMessage message = activity.getMessage();
        assertEquals("Is not a message " + activity, MbObjectType.MESSAGE, activity.getObjectType());
        assertEquals("Favorited " + activity, TriState.UNKNOWN, activity.getMessage().getFavoritedBy(activity.accountUser));
        MbUser actor = activity.getActor();

        String stringDate = "2017-04-16T11:13:12.133Z";
        long parsedDate = connection.parseDate(stringDate);
        assertEquals("Parsing " + stringDate, 4, new Date(parsedDate).getMonth() + 1);
        assertEquals("Created at", parsedDate, actor.getCreatedDate());

        assertEquals("Actor is partially defined " + actor, false, actor.isPartiallyDefined());
        assertEquals("Actor Oid", "37", actor.oid);
        assertEquals("Username", "t131t1", actor.getUserName());

        assertEquals("Message Oid " + activity, "22", message.oid);
        assertEquals("Message url" + activity, "https://neumastodon.com/@t131t1/22", message.url);
        assertEquals("Body", "<p>I'm figuring out how to work with Mastodon</p>", message.getBody());
        assertEquals("Message application", "Web", message.via);

        assertEquals("Media attachments", 1, message.attachments.size());
        MbAttachment attachment = message.attachments.get(0);
        assertEquals("Content type", MyContentType.IMAGE, attachment.contentType);
        assertEquals("Content type", UriUtils.fromString("https://files.neumastodon.com/media_attachments/files/000/306/223/original/e678f956970a585b.png?1492832537"),
                attachment.getUri());
    }

    @Test
    public void testGetConversation() throws IOException {
        String jso = RawResourceUtils.getString(InstrumentationRegistry.getInstrumentation().getContext(),
                org.andstatus.app.tests.R.raw.mastodon_get_conversation);
        connection.getHttpMock().setResponse(jso);

        List<MbActivity> timeline = connection.getConversation("5596683");
        assertNotNull("timeline returned", timeline);
        assertEquals("Number of items in the Timeline", 5, timeline.size());
    }

    @Test
    public void testGetNotifications() throws IOException {
        String jso = RawResourceUtils.getString(InstrumentationRegistry.getInstrumentation().getContext(),
                org.andstatus.app.tests.R.raw.mastodon_notifications);
        connection.getHttpMock().setResponse(jso);

        List<MbActivity> timeline = connection.getTimeline(Connection.ApiRoutineEnum.NOTIFICATIONS_TIMELINE,
                new TimelinePosition(""), TimelinePosition.EMPTY, 20, accountUserOid);
        assertNotNull("timeline returned", timeline);
        assertEquals("Number of items in the Timeline", 20, timeline.size());

        int ind = 0;
        MbActivity activity = timeline.get(ind);
        assertEquals("Timeline position", "2667058", activity.getTimelinePosition().getPosition());
        assertEquals("Message Oid", "4729037", activity.getMessage().oid);
        assertEquals("Is not a Reblog " + activity, MbActivityType.ANNOUNCE, activity.type);
        assertEquals("Is not an activity", MbObjectType.ACTIVITY, activity.getObjectType());
        MbUser actor = activity.getActor();
        assertEquals("Actor's Oid", "15451", actor.oid);
        assertEquals("Actor's username", "Chaosphere", actor.getUserName());
        assertEquals("WebfingerId", "Chaosphere@mastodon.social", actor.getWebFingerId());
        assertEquals("Author's username" + activity, "AndStatus", activity.getAuthor().getUserName());
        assertEquals("Favorited " + activity, TriState.UNKNOWN, activity.getMessage().getFavoritedBy(activity.accountUser));

        ind = 2;
        activity = timeline.get(ind);
        assertEquals("Timeline position", "2674022", activity.getTimelinePosition().getPosition());
        assertEquals("Message Oid", "4729037", activity.getMessage().oid);
        assertEquals("Is not an activity " + activity, MbObjectType.ACTIVITY, activity.getObjectType());
        assertEquals("Is not LIKE " + activity, MbActivityType.LIKE, activity.type);
        assertThat(activity.getMessage().getBody(), is("<p>IT infrastructure of modern church</p>"));
        assertEquals("Favorited " + activity, TriState.UNKNOWN, activity.getMessage().getFavoritedBy(activity.accountUser));
        assertEquals("Author's username", "AndStatus", activity.getAuthor().getUserName());
        actor = activity.getActor();
        assertEquals("Actor's Oid", "48790", actor.oid);
        assertEquals("Actor's Username", "vfrmedia", actor.getUserName());
        assertEquals("WebfingerId", "vfrmedia@social.tchncs.de", actor.getWebFingerId());

        ind = 17;
        activity = timeline.get(ind);
        assertEquals("Is not FOLLOW " + activity, MbActivityType.FOLLOW, activity.type);
        assertEquals("Is not a USER", MbObjectType.USER, activity.getObjectType());
        actor = activity.getActor();
        assertEquals("Actor's Oid", "24853", actor.oid);
        assertEquals("Username", "resir014", actor.getUserName());
        assertEquals("WebfingerId", "resir014@icosahedron.website", actor.getWebFingerId());
        MbUser user = activity.getUser();
        assertEquals("Not following me" + activity, accountUserOid, user.oid);

        ind = 19;
        activity = timeline.get(ind);
        assertEquals("Is not UPDATE " + activity, MbActivityType.UPDATE, activity.type);
        assertEquals("Is not a message", MbObjectType.MESSAGE, activity.getObjectType());
        assertThat(activity.getMessage().getBody(), containsString("universe of Mastodon"));
        actor = activity.getActor();
        assertEquals("Actor's Oid", "119218", actor.oid);
        assertEquals("Username", "izwx6502", actor.getUserName());
        assertEquals("WebfingerId", "izwx6502@mstdn.jp", actor.getWebFingerId());
    }
}
